@@ -1,15 +1,15 @@
 # Copyright (c) Ye Liu. All rights reserved.
 
 from .buffer import Buffer
-from .comm import (all_gather, gather, get_dist_info, get_rank, get_world_size,
-                   init_dist, is_distributed, is_main_process, master_only,
-                   synchronize)
+from .comm import (all_gather, broadcast, gather, get_dist_info, get_rank,
+                   get_world_size, init_dist, is_distributed, is_main_process,
+                   master_only, synchronize)
 from .engine import Engine
 from .utils import (generate_random_seed, get_checkpoint, load_checkpoint,
                     save_checkpoint, set_random_seed)
 
 __all__ = [
-    'Buffer', 'all_gather', 'gather', 'get_dist_info', 'get_rank',
+    'Buffer', 'all_gather', 'broadcast', 'gather', 'get_dist_info', 'get_rank',
     'get_world_size', 'init_dist', 'is_distributed', 'is_main_process',
     'master_only', 'synchronize', 'Engine', 'generate_random_seed',
     'get_checkpoint', 'load_checkpoint', 'save_checkpoint', 'set_random_seed'
