@@ -9,7 +9,7 @@ from .losses import (FocalLoss, FocalLossStar, GHMCLoss, sigmoid_focal_loss,
                      sigmoid_focal_loss_star)
 from .modules import (LinearModule, MsgPassModule, build_mlp,
                       build_msg_pass_modules)
-from .utils import fuse_conv_bn, publish_model, update_bn_stats
+from .utils import fuse_bn_, move_to_device, publish_model, update_bn_stats_
 
 __all__ = [
     'ACTIVATIONS', 'CONVS', 'GAT', 'MESSAGE_PASSINGS', 'NORMS', 'Clamp',
@@ -18,6 +18,6 @@ __all__ = [
     'kaiming_init_', 'normal_init_', 'uniform_init_', 'xavier_init_',
     'FocalLoss', 'FocalLossStar', 'GHMCLoss', 'sigmoid_focal_loss',
     'sigmoid_focal_loss_star', 'LinearModule', 'MsgPassModule', 'build_mlp',
-    'build_msg_pass_modules', 'fuse_conv_bn', 'publish_model',
-    'update_bn_stats'
+    'build_msg_pass_modules', 'fuse_bn_', 'move_to_device', 'publish_model',
+    'update_bn_stats_'
 ]
