@@ -1,16 +1,14 @@
 # Copyright (c) Ye Liu. All rights reserved.
 
 import os
-import os.path as osp
 import sys
 from collections import OrderedDict
 from copy import deepcopy
 from importlib import import_module
-from shutil import copyfile
 from tempfile import TemporaryDirectory
 
 import nncore
-from .binder import bind_getter, bind_method
+from .binder import bind_getter
 
 
 class CfgNode(OrderedDict):
@@ -38,14 +36,14 @@ class CfgNode(OrderedDict):
             raise TypeError('too many arguments')
 
         if len(args) == 1:
-            if isinstance(args[0], (Config, dict)):
+            if isinstance(args[0], dict):
                 kwargs.update(args[0])
             else:
                 raise TypeError("unsupported type '{}'".format(type(args[0])))
 
         super(CfgNode, self).__setattr__('_frozen', False)
-        for k, v in kwargs.items():
-            self[k] = v
+        for key, value in kwargs.items():
+            self[key] = value
 
     def __setitem__(self, key, value):
         self._check_freeze_state()
@@ -64,10 +62,22 @@ class CfgNode(OrderedDict):
         self._set_freeze_state(self, state['_frozen'])
 
     def __deepcopy__(self, memo):
+
+        def _copy(obj, memo):
+            try:
+                return deepcopy(obj, memo)
+            except TypeError:
+                return obj
+
         other = self.__class__()
         memo[id(self)] = other
-        for k, v in self.items():
-            other[deepcopy(k, memo)] = deepcopy(v, memo)
+        for key, value in self.items():
+            key = _copy(key, memo)
+            if isinstance(value, (list, tuple)):
+                value = type(value)(_copy(v, memo) for v in value)
+            else:
+                value = _copy(value, memo)
+            other[key] = value
         return other
 
     def __eq__(self, other):
@@ -90,8 +100,6 @@ class CfgNode(OrderedDict):
             value = self.__class__(**value)
         elif isinstance(value, (list, tuple)):
             value = type(value)(self._parse_value(v) for v in value)
-        elif isinstance(value, range):
-            value = list(self._parse_value(v) for v in value)
         return value
 
     def _check_freeze_state(self):
@@ -114,23 +122,49 @@ class CfgNode(OrderedDict):
         elif len(args) > 1:
             raise TypeError('too many arguments')
         other.update(kwargs)
-        for k, v in other.items():
-            if ((k not in self) or (not isinstance(self[k], dict))
-                    or (not isinstance(v, dict))):
-                self[k] = self._parse_value(v)
+        for key, value in other.items():
+            if key not in self or not isinstance(
+                    self[key], dict) or not isinstance(value, dict):
+                self[key] = self._parse_value(value)
             else:
-                self[k].update(v)
+                self[key].update(value)
+
+    def merge_from(self, other):
+        assert isinstance(other, dict)
+        other = self.__class__(**other)
+
+        for key, value in other.items():
+            if isinstance(value, dict):
+                delete = value.pop('_delete_', 0)
+                append = value.pop('_append_', 0)
+                assert not delete or not append
+
+                if key in self and isinstance(
+                        self[key], dict) and not delete and not append:
+                    self[key].merge_from(value)
+                elif key in self and append:
+                    if isinstance(self[key], list):
+                        self[key].append(value)
+                    elif isinstance(self[key], tuple):
+                        self[key] = tuple(list(self[key]) + [value])
+                    else:
+                        self[key] = [self[key]] + [value]
+                else:
+                    self[key] = value
+            else:
+                self[key] = value
 
     def to_dict(self, ordered=False):
         base = OrderedDict() if ordered else dict()
-        for k, v in self.items():
-            if isinstance(v, type(self)):
-                base[k] = v.to_dict()
-            elif isinstance(v, (list, tuple)):
-                base[k] = type(v)(
-                    o.to_dict() if isinstance(o, type(self)) else o for o in v)
+        for key, value in self.items():
+            if isinstance(value, self.__class__):
+                base[key] = value.to_dict()
+            elif isinstance(value, (list, tuple)):
+                base[key] = type(value)(
+                    o.to_dict() if isinstance(o, self.__class__) else o
+                    for o in value)
             else:
-                base[k] = v
+                base[key] = value
         return base
 
     def to_json(self):
@@ -138,15 +172,11 @@ class CfgNode(OrderedDict):
 
 
 @bind_getter('filename')
-@bind_method('_cfg', [
-    '__getitem__', '__setitem__', '__len__', '__iter__', '__eq__', 'get',
-    'pop', 'freeze', 'unfreeze', 'setdefault', 'to_dict'
-])
-class Config(object):
+class Config(CfgNode):
     """
     A facility for better :obj:`CfgNode` objects.
 
-    This class is a wrapper for :obj:`CfgNode` which can be initialized from a
+    This class inherits from :obj:`CfgNode` and it can be initialized from a
     config file. Users can use the static method :obj:`Config.from_file` to
     create a :obj:`Config` object.
     """
@@ -165,16 +195,15 @@ class Config(object):
         Returns:
             :obj:`Config`: The constructed config object.
         """
-        filename = nncore.abs_path(osp.expanduser(filename))
-        nncore.file_exist(filename, raise_error=True)
+        filename = nncore.abs_path(filename)
+        nncore.is_file(filename, raise_error=True)
 
-        format = filename.split('.')[-1]
+        format = nncore.pure_ext(filename)
         if format == 'py':
-            with TemporaryDirectory() as tmp_dir:
+            with TemporaryDirectory() as tmp:
                 mod_name = str(int.from_bytes(os.urandom(2), 'big'))
-                copyfile(filename,
-                         nncore.join(tmp_dir, '{}.py'.format(mod_name)))
-                sys.path.insert(0, tmp_dir)
+                nncore.cp(filename, nncore.join(tmp, '{}.py'.format(mod_name)))
+                sys.path.insert(0, tmp)
                 mod = import_module(mod_name)
                 sys.path.pop(0)
                 cfg = {
@@ -187,101 +216,96 @@ class Config(object):
         else:
             raise TypeError("unsupported format: '{}'".format(format))
 
-        return Config(cfg=cfg, filename=filename, freeze=freeze)
+        if '_base_' in cfg:
+            base = cfg.pop('_base_')
+            if isinstance(base, str):
+                base = [base]
 
-    def __init__(self, cfg=None, filename=None, freeze=False):
-        if isinstance(cfg, (type(self), dict)):
-            _cfg = CfgNode(cfg)
-        elif cfg is None:
             _cfg = CfgNode()
-        else:
-            raise TypeError("unsupported type: '{}'".format(type(cfg)))
+            for name in base:
+                path = nncore.join(nncore.dir_name(filename), name)
+                _cfg.merge_from(Config.from_file(path))
 
-        super(Config, self).__setattr__('_cfg', _cfg)
-        super(Config, self).__setattr__('_filename', filename)
+            _cfg.merge_from(cfg)
+            cfg = _cfg
+
+        return Config(cfg, filename=filename, freeze=freeze)
+
+    def __init__(self, *args, filename=None, freeze=False, **kwargs):
+        super(Config, self).__init__(*args, **kwargs)
+        super(CfgNode, self).__setattr__('_filename', filename)
 
         if freeze:
             self.freeze()
 
-        if filename is not None:
-            with open(filename, 'r') as f:
-                super(Config, self).__setattr__('_text', f.read())
-        else:
-            super(Config, self).__setattr__('_text', None)
+    def __repr__(self):
+        return '{}({}frozen={}): {}'.format(
+            self.__class__.__name__, '' if self._filename is None else
+            "filename='{}', ".format(self._filename), self._frozen,
+            super(Config, self).__repr__())
 
     @property
     def text(self):
-        text = '' if self._filename is None else '{}\n'.format(self._filename)
 
-        if self._text is not None:
-            return text + self._text
-
-        def _indent(attr_str):
-            tokens = attr_str.split('\n')
+        def _indent(a_str):
+            tokens = a_str.split('\n')
             if len(tokens) == 1:
-                return attr_str
+                return a_str
             first = tokens.pop(0)
             tokens = [' ' * 4 + line for line in tokens]
             return '{}\n{}'.format(first, '\n'.join(tokens))
 
-        def _basic(key, value):
+        def _basic(key, value, blank=True):
+            base_str = '{} = {}' if blank else '{}={}'
             if isinstance(value, dict):
                 v_str = _dict(value)
             elif isinstance(value, str):
                 v_str = "'{}'".format(value)
             else:
                 v_str = str(value)
-            attr_str = v_str if key is None else '{} = {}'.format(key, v_str)
-            return _indent(attr_str)
+            a_str = v_str if key is None else base_str.format(key, v_str)
+            return _indent(a_str)
 
-        def _iterable(key, value):
-            tokens = []
+        def _iterable(key, value, blank=True):
+            base_str, tokens = '{} = {}' if blank else '{}={}', []
+            prefix = '\n' if len(value) > 1 else ''
+            expand = any(isinstance(v, (dict, list, tuple)) for v in value)
             for v in value:
                 if isinstance(v, dict):
-                    tokens.append('dict({})'.format(_indent('\n' + _dict(v))))
+                    if len(v) > 1:
+                        a_str = _indent('\n' + _dict(v))
+                    else:
+                        a_str = _dict(v)
+                    tokens.append(_indent(prefix + 'dict({})'.format(a_str)))
                 elif isinstance(v, (list, tuple)):
-                    tokens.append(_iterable(None, v))
+                    tokens.append(_indent(prefix + _iterable(None, v)))
                 else:
-                    tokens.append(_basic(None, v))
+                    a_str = _basic(None, v)
+                    tokens.append(_indent(prefix + a_str) if expand else a_str)
             left, right = ('[', ']') if isinstance(value, list) else ('(', ')')
-            v_str = '{}{}{}'.format(left, ', '.join(tokens), right)
-            return v_str if key is None else '{} = {}'.format(key, v_str)
+            sep = ',' if expand else ', '
+            v_str = '{}{}{}'.format(left, sep.join(tokens), right)
+            return v_str if key is None else base_str.format(key, v_str)
 
         def _dict(value, parent=False):
-            tokens = []
-            for idx, (k, v) in enumerate(value.items()):
-                is_last = idx >= len(value) - 1
-                end = '' if parent or is_last else ','
+            base_str, tokens = '{} = dict({})' if parent else '{}=dict({})', []
+            for i, (k, v) in enumerate(value.items()):
+                end = '' if parent or i >= len(value) - 1 else ','
                 if isinstance(v, dict):
-                    v_str = '\n' + _dict(v)
-                    attr_str = '{} = dict({}'.format(str(k), v_str)
-                    attr_str = _indent(attr_str) + ')' + end
+                    if len(v) > 1:
+                        a_str = base_str.format(str(k), '\n' + _dict(v))
+                        a_str = _indent(a_str) + end
+                    else:
+                        a_str = base_str.format(str(k), _dict(v)) + end
                 elif isinstance(v, (list, tuple)):
-                    attr_str = _iterable(k, v) + end
+                    a_str = _iterable(k, v, blank=parent) + end
                 else:
-                    attr_str = _basic(k, v) + end
-                tokens.append(attr_str)
+                    a_str = _basic(k, v, blank=parent) + end
+                tokens.append(a_str)
             return '\n'.join(tokens)
 
-        cfg_dict = self._cfg.to_dict()
-        text += _dict(cfg_dict, parent=True)
+        text = _dict(self.to_dict(), parent=True)
+        if self._filename is not None:
+            text = '{}\n'.format(self._filename) + text
 
         return text
-
-    def __getattr__(self, key):
-        return getattr(self._cfg, key)
-
-    def __setattr__(self, key, value):
-        setattr(self._cfg, key, value)
-
-    def __repr__(self):
-        return '{}{}: {}'.format(
-            self.__class__.__name__, '' if self._filename is None else
-            "(filename: '{}')".format(self._filename), self._cfg)
-
-    def update(self, *args, **kwargs):
-        args = [arg._cfg if isinstance(arg, Config) else arg for arg in args]
-        self._cfg.update(*args, **kwargs)
-
-    def copy(self):
-        return deepcopy(self)
